<?php

namespace Drupal\entity_count_tokens\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Query\Query;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Url;
use Drupal\Core\Utility\TableSort;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Entity count controller.
 */
class EntityCountTokensController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type bundle info.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * The constructor.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    EntityTypeBundleInfoInterface $entity_type_bundle_info,
    Renderer $renderer
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('renderer')
    );
  }

  /**
   * Get the entity count raw report.
   *
   * @return array
   *   Returns the raw report.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function entityCountRaw(): array {
    $rows = [];

    $definitions = $this->entityTypeManager->getDefinitions();
    foreach ($definitions as $definition) {
      $definition_id = $definition->id();

      $storage = $this->entityTypeManager->getStorage($definition_id);
      if (get_class($definition) == 'Drupal\Core\Config\Entity\ConfigEntityType') {
        $entities = $storage->loadMultiple();
        $count = count($entities);
      }
      else {
        $query = $storage->getAggregateQuery();
        $query->accessCheck(TRUE);
        $query->count();
        $count = $query->execute();
      }

      $bundle_info = $this->entityTypeBundleInfo->getBundleInfo($definition_id);

      if (count($bundle_info) > 1) {
        $bundle_info = $this->entityTypeBundleInfo->getBundleInfo($definition_id);
        foreach ($bundle_info as $bundle_id => $bundle_data) {
//          $definition = $this->entityTypeManager->getDefinition($definition_id);

          $keys = $definition->getKeys();
          $bundle_key = is_null($keys['bundle']) ? 'bundle' : $keys['bundle'];

          $query = $this->entityTypeManager->getStorage($definition->id())->getQuery();
          $query->condition($bundle_key, $bundle_id);
          $query->accessCheck(TRUE);
          $query->count();
          $count = $query->execute();

          $rows[$definition_id][$bundle_id] = $count;
        }
      }
      else {
        $rows[$definition_id] = $count;
      }
    }
    return $rows;
  }


  /**
   * Generate all tokens
   *
   * @return array
   *   Returns all possible tokens in the website
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function generateAllTokens(): array {
    $rows = [];

    $definitions = $this->entityTypeManager->getDefinitions();
    foreach ($definitions as $definition) {
      $definition_id = $definition->id();
/****
      $storage = $this->entityTypeManager->getStorage($definition_id);
      if (get_class($definition) == 'Drupal\Core\Config\Entity\ConfigEntityType') {
        $entities = $storage->loadMultiple();
        $count = count($entities);
      }
      else {
        $query = $storage->getAggregateQuery();
        $query->accessCheck(TRUE);
        $query->count();
        $count = $query->execute();
      }
****/
      $bundle_info = $this->entityTypeBundleInfo->getBundleInfo($definition_id);

      if (count($bundle_info) > 1) {
        $bundle_info = $this->entityTypeBundleInfo->getBundleInfo($definition_id);
        foreach ($bundle_info as $bundle_id => $bundle_data) {
//          $definition = $this->entityTypeManager->getDefinition($definition_id);

          $keys = $definition->getKeys();
          $bundle_key = is_null($keys['bundle']) ? 'bundle' : $keys['bundle'];
/*
          $query = $this->entityTypeManager->getStorage($definition->id())->getQuery();
          $query->condition($bundle_key, $bundle_id);
          $query->accessCheck(TRUE);
          $query->count();
          $count = $query->execute();
*/
          $rows[] = "entity_count_tokens:$definition_id:$bundle_id";
        }
      }
      else {
        $rows[] = "entity_count_tokens.$definition_id";
      }
    }

    return $rows;
  }


}
